package jp.agentec.sinaburocast.logic;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.internet.InternetAddress;
import javax.transaction.UserTransaction;

import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.exception.ExpectedException;
import jp.agentec.sinaburocast.common.util.MailUtil;
import jp.agentec.sinaburocast.common.util.PropertyUtil;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.entity.Member;
import jp.agentec.sinaburocast.helper.KeyHelper;
import jp.agentec.sinaburocast.service.MemberService;
import jp.agentec.sinaburocast.service.MessageService;
import jp.agentec.sinaburocast.vo.EmailKey;

import org.apache.commons.lang.RandomStringUtils;
import org.apache.log4j.Logger;
import org.seasar.framework.container.annotation.tiger.Component;
import org.seasar.framework.container.annotation.tiger.InstanceType;

@Component(instance=InstanceType.SINGLETON)
public class MemberRegisterBatchLogic {
	private final Logger logger = Logger.getLogger(getClass());

	public MemberService memberService;
	public MessageService messageService;

	public KeyHelper keyHelper;

	protected Map requestScope;

	@TransactionAttribute(TransactionAttributeType.NEVER)
	public void emptyMail(String insId,Map requestScope) {
		
		try {
			this.requestScope = requestScope;

			String pop3host = PropertyUtil.getProperty("SMTP_HOST");
			String pop3userid = PropertyUtil.getProperty("REGISTER_MAIL_USER");
			String pop3pass = PropertyUtil.getProperty("REGISTER_MAIL_PASS");

			Properties props = System.getProperties();
			Session session = Session.getDefaultInstance(props, null);
			session.setDebug(PropertyUtil.getBoolean("JAVA_MAIL_DEBUG"));
			Store store = session.getStore("pop3");
			store.connect(pop3host, pop3userid, pop3pass);

			Folder folder = null;

			folder = store.getFolder("INBOX");

			try{
				folder.open(Folder.READ_WRITE);

				// メッセージ一覧を取得
				final Message messages[] = folder.getMessages();

				for (int index = 0; index < messages.length; index++) {
					final Message message = messages[index];

					try {
						if (message.getFrom() == null || message.getFrom().length == 0) {
							logger.warn("No From header. Ignored.");
							message.setFlag(Flags.Flag.DELETED, true);
							continue;
						}

						final InternetAddress addrFrom = (InternetAddress) message.getFrom()[0];

						if (addrFrom.getAddress().indexOf("MAILER-DAEMON") >= 0) {
							logger.warn("Has MAILER-DAEMON. Ignored.");
							message.setFlag(Flags.Flag.DELETED, true);
							continue;
						}

						// 登録確認メール送信
						sendRegisterMail(addrFrom.getAddress());
						message.setFlag(Flags.Flag.DELETED, true);
					} catch (Exception e) {
						// エラーメール削除
						logger.fatal("Mail check failed ", e);
						message.setFlag(Flags.Flag.DELETED, true);
					}
				}
			} catch (Exception e) {
				logger.fatal("Mail check failed ", e);
			} finally {
				folder.close(true);
			}
		} catch (Exception e) {
			logger.error("Mail check failed ", e);
		}

	}

	private void sendRegisterMail(String email) throws Exception {
		Member member;
		if (SinaburoUtil.mobileAddressCheck(email)) {
			member = memberService.findBySubPcEmail(email);
		} else {
			member = memberService.findByMainPcEmail(email);
		}
		
		if (member != null) { // すでに登録されている場合のメール送信
			memberService.memberRegisterd(member, email, PropertyUtil.getString("ENQUETE_SERVER_URL"));
		}
		else { // 暗号化キーを生成して登録確認メールを送る
			String registerKey = keyHelper.encryptEmail(email, 3, 0, 0); // リンクの有効期限固定3か月
			jp.agentec.sinaburocast.entity.Message message = messageService.findById(SinaburoConstant.MessageCode.MEMBER_REGISTER_CONFIRM);
			//メール送信
			String body= MessageFormat.format(message.message.replace("\\r\\n", "\r\n"), registerKey);
			MailUtil.send(email, PropertyUtil.getProperty("REGISTER_CONFIRM_MAIL_TITLE"), body);
		}
	}

	/**
	 * 登録を行う。
	 * メール送信
	 * @return 
	 *
	 * @return
	 */
	public Member regist(String email) throws Exception {

		Member member = new Member();

		Member memberInfo = null;
		List<Member> memberList = new ArrayList<Member>();

		if (SinaburoUtil.mobileAddressCheck(email)) {
			member.subPcEmail = email;
			memberInfo = memberService.findBySubPcEmail(email);
		} else {
			member.mainPcEmail = email;
			memberInfo = memberService.findByMainPcEmail(email);
		}

		if (memberInfo == null) {
		//未登録済のメールアドレス
			String id = RandomStringUtils.randomAlphanumeric(6);

			//自動生成のメンバID重複チェック
			while (true) {
				memberList = memberService.findByLoginId(id);
				if (memberList.size() != 0) {
					id = RandomStringUtils.randomAlphanumeric(6);
				} else {
					break;
				}
			}

			member.loginId = id;
			String pass = RandomStringUtils.randomAlphanumeric(4);
			member.password = pass;
			member.validFlg = 1;
			member.pointNum = 0;
			member.authFailNum = 0;
			member.delFlg = 0;

			String url3 = PropertyUtil.getString("ENQUETE_SERVER_URL");

			//メール送信、データ登録
			try {
				int retValue = memberService.insertRegistMailSendLogic(member,email,url3);
			} catch (MessagingException e) {
				logger.error("Mail send failed." + e.toString()); // 例外は投げない
			}
			
			member.isNew = true;
			return member;
		}else{

			//メール送信のみ
			//メール送信、データ登録
			String url3 = PropertyUtil.getString("ENQUETE_SERVER_URL");
			try {
				memberService.memberRegisterd(memberInfo,email,url3);
			} catch (MessagingException e) {
				logger.error("Mail send failed." + e.toString()); // 例外は投げない
			}
			
			memberInfo.isNew = false;
			return memberInfo;
		}
	}

	public Member registByKey(String registerKey) throws Exception {
		EmailKey emailKey;
		try {
			emailKey = keyHelper.decryptEmail(registerKey);
		} catch (Exception e) {
			logger.error("registByKey error. " + e.toString());
			throw new ExpectedException("", "errors.E061");
		}
		if (emailKey.expired.before(new Date())) {
			throw new ExpectedException("", "errors.E062");
		}
		if (!SinaburoUtil.emailCheck(emailKey.email)) {
			throw new ExpectedException("", "errors.E061");
		}
		return regist(emailKey.email);
	}

}
